@@ -13,6 +13,7 @@
 #   under the License.
 #
 
+import fixtures
 import mock
 import os
 import tempfile
@@ -362,9 +363,53 @@ class TestDeployUndercloud(TestPluginV1):
                                                      env_files)
         self.assertEqual(expected, results)
 
-    @mock.patch('os.path.exists', return_value=False)
+    @mock.patch('yaml.safe_load', return_value={}, autospec=True)
+    @mock.patch('yaml.safe_dump', autospec=True)
+    @mock.patch('os.path.isfile', return_value=True)
+    @mock.patch('six.moves.builtins.open')
     @mock.patch('tripleoclient.v1.tripleo_deploy.Deploy.'
-                '_create_working_dirs')
+                '_process_hieradata_overrides', autospec=True)
+    @mock.patch('tripleoclient.v1.tripleo_deploy.Deploy.'
+                '_update_passwords_env', autospec=True)
+    @mock.patch('tripleoclient.v1.tripleo_deploy.Deploy.'
+                '_normalize_user_templates', return_value=[], autospec=True)
+    @mock.patch('tripleoclient.utils.rel_or_abs_path', return_value={},
+                autospec=True)
+    @mock.patch('tripleoclient.utils.run_command_and_log', return_value=0,
+                autospec=True)
+    def test_setup_heat_environments_dropin(
+            self, mock_run, mock_paths, mock_norm, mock_update_pass_env,
+            mock_process_hiera, mock_open, mock_os, mock_yaml_dump,
+            mock_yaml_load):
+
+        parsed_args = self.check_parser(self.cmd,
+                                        ['--local-ip', '127.0.0.1/8',
+                                         '--templates', 'tht_from',
+                                         '--output-dir', 'tht_to'], [])
+        dropin = 'tht_from/standalone-stack-vstate-dropin.yaml'
+        self.cmd.output_dir = 'tht_to'
+        self.cmd.tht_render = 'tht_from'
+        self.cmd.stack_action = 'UPDATE'
+        environment = self.cmd._setup_heat_environments(parsed_args)
+
+        self.assertIn(dropin, environment)
+        mock_open.assert_has_calls([mock.call(dropin, 'w')])
+
+        # unpack the dump yaml calls to verify if the produced stack update
+        # dropin matches our expectations
+        found_dropin = False
+        for call in mock_yaml_dump.call_args_list:
+            args, kwargs = call
+            for a in args:
+                if isinstance(a, mock.mock.MagicMock):
+                    continue
+                if a.get('parameter_defaults', {}).get('StackAction', None):
+                    self.assertTrue(
+                        a['parameter_defaults']['StackAction'] == 'UPDATE')
+                    found_dropin = True
+        self.assertTrue(found_dropin)
+
+    @mock.patch('os.path.isfile')
     @mock.patch('heatclient.common.template_utils.'
                 'process_environment_and_files', return_value=({}, {}),
                 autospec=True)
@@ -374,68 +419,127 @@ class TestDeployUndercloud(TestPluginV1):
     @mock.patch('tripleoclient.utils.'
                 'process_multiple_environments', autospec=True)
     @mock.patch('tripleoclient.v1.tripleo_deploy.Deploy.'
-                '_process_hieradata_overrides', return_value='foo.yaml',
+                '_process_hieradata_overrides', return_value='hiera_or.yaml',
                 autospec=True)
     @mock.patch('tripleoclient.v1.tripleo_deploy.Deploy.'
                 '_update_passwords_env', autospec=True)
     @mock.patch('tripleoclient.utils.'
                 'run_command_and_log', autospec=True)
-    @mock.patch('six.moves.builtins.open')
-    @mock.patch('shutil.copy', autospec=True)
-    def test_setup_heat_environments(self, mock_cp, mock_open,
-                                     mock_run,
-                                     mock_update_pass_env,
-                                     mock_process_hiera,
-                                     mock_process_multiple_environments,
-                                     mock_hc_get_templ_cont,
-                                     mock_hc_process,
-                                     mock_createdirs,
-                                     mock_exists):
+    def test_setup_heat_environments_default_plan_env(
+            self, mock_run, mock_update_pass_env, mock_process_hiera,
+            mock_process_multiple_environments, mock_hc_get_templ_cont,
+            mock_hc_process, mock_os):
 
-        mock_update_pass_env.return_value = '/my/tripleo-heat-installer-' \
-                                            'templates/passwords.yaml'
+        tmpdir = self.useFixture(fixtures.TempDir()).path
+        tht_from = os.path.join(tmpdir, 'tht-from')
+        os.mkdir(tht_from)
+        plan_env_path = os.path.join(tht_from, 'plan-environment.yaml')
+        with open(plan_env_path, mode='w') as plan_file:
+            yaml.dump({'environments': [{'path': 'env.yaml'}]}, plan_file)
+        self.assertTrue(os.path.exists(plan_env_path))
+        self._setup_heat_environments(tmpdir, tht_from, plan_env_path,
+                                      mock_update_pass_env, mock_run)
+
+    @mock.patch('os.path.isfile')
+    @mock.patch('heatclient.common.template_utils.'
+                'process_environment_and_files', return_value=({}, {}),
+                autospec=True)
+    @mock.patch('heatclient.common.template_utils.'
+                'get_template_contents', return_value=({}, {}),
+                autospec=True)
+    @mock.patch('tripleoclient.utils.'
+                'process_multiple_environments', autospec=True)
+    @mock.patch('tripleoclient.v1.tripleo_deploy.Deploy.'
+                '_process_hieradata_overrides', return_value='hiera_or.yaml',
+                autospec=True)
+    @mock.patch('tripleoclient.v1.tripleo_deploy.Deploy.'
+                '_update_passwords_env', autospec=True)
+    @mock.patch('tripleoclient.utils.'
+                'run_command_and_log', autospec=True)
+    def test_setup_heat_environments_non_default_plan_env(
+            self, mock_run, mock_update_pass_env, mock_process_hiera,
+            mock_process_multiple_environments, mock_hc_get_templ_cont,
+            mock_hc_process, mock_os):
+
+        tmpdir = self.useFixture(fixtures.TempDir()).path
+        tht_from = os.path.join(tmpdir, 'tht-from')
+        os.mkdir(tht_from)
+        default_plan_env_path = os.path.join(tht_from, 'plan-environment.yaml')
+        with open(default_plan_env_path, mode='w') as plan_file:
+            yaml.dump({'environments': [{'path': 'env.yaml'}]}, plan_file)
+        plan_env_path = os.path.join(tmpdir, 'plan-environment.yaml')
+        with open(plan_env_path, mode='w') as plan_file:
+            yaml.dump({'environments': [{'path': 'notenv.yaml'}]}, plan_file)
+        self.assertTrue(os.path.exists(plan_env_path))
+        with open(os.path.join(tht_from, 'notenv.yaml'),
+                  mode='w') as env_file:
+            yaml.dump({}, env_file)
+        cmd_extra = ['-p', plan_env_path]
+        self._setup_heat_environments(tmpdir, tht_from, plan_env_path,
+                                      mock_update_pass_env, mock_run,
+                                      cmd_extra, 'notenv.yaml')
+
+    def _setup_heat_environments(self, tmpdir, tht_from, plan_env_path,
+                                 mock_update_pass_env, mock_run,
+                                 extra_cmd=None, plan_env_env=None):
+        cmd_extra = extra_cmd or []
+        plan_env_env_name = plan_env_env or 'env.yaml'
+
+        tht_outside = os.path.join(tmpdir, 'tht-outside')
+        os.mkdir(tht_outside)
+        tht_to = os.path.join(tmpdir, 'tht-to')
+        os.mkdir(tht_to)
+        with open(os.path.join(tht_from, 'env.yaml'),
+                  mode='w') as env_file:
+            yaml.dump({}, env_file)
+        with open(os.path.join(tht_from, 'foo.yaml'),
+                  mode='w') as env_file:
+            yaml.dump({}, env_file)
+        with open(os.path.join(tht_outside, 'outside.yaml'),
+                  mode='w') as env_file:
+            yaml.dump({}, env_file)
+
+        tht_render = os.path.join(tht_to, 'tripleo-heat-installer-templates')
+        mock_update_pass_env.return_value = os.path.join(
+            tht_render, 'passwords.yaml')
         mock_run.return_value = 0
 
-        # logic handled in _standalone_deploy and _create_working_dirs
-        self.cmd.output_dir = '/my'
-        self.cmd.tht_render = '/my/tripleo-heat-installer-templates'
+        # logic handled in _standalone_deploy
+        self.cmd.output_dir = tht_to
+        # Note we don't create tht_render as _populate_templates_dir creates it
+        self.cmd.tht_render = tht_render
+        self.cmd._populate_templates_dir(tht_from)
 
         parsed_args = self.check_parser(self.cmd,
                                         ['--local-ip', '127.0.0.1/8',
-                                         '--templates', '/tmp/thtroot',
-                                         '--output-dir', '/my',
+                                         '--templates', tht_from,
+                                         '--output-dir', tht_to,
                                          '--hieradata-override',
                                          'legacy.yaml',
                                          '-e',
-                                         '/tmp/thtroot/puppet/foo.yaml',
+                                         os.path.join(tht_from, 'foo.yaml'),
                                          '-e',
-                                         '/tmp/thtroot//docker/bar.yaml',
-                                         '-e',
-                                         '/tmp/thtroot42/notouch.yaml',
-                                         '-e', '~/custom.yaml',
-                                         '-e', 'something.yaml',
-                                         '-e', '../../../outside.yaml'], [])
+                                         os.path.join(tht_outside,
+                                                      'outside.yaml'),
+                                         ] + cmd_extra, [])
         expected_env = [
-            '/my/tripleo-heat-installer-templates/'
-            'overcloud-resource-registry-puppet.yaml',
-            '/my/tripleo-heat-installer-templates/passwords.yaml',
-            '/my/tripleo-heat-installer-templates/'
-            'environments/config-download-environment.yaml',
-            '/my/tripleo-heat-installer-templates/'
-            'environments/deployed-server-noop-ctlplane.yaml',
-            '/my/tripleo-heat-installer-templates/'
-            'tripleoclient-hosts-portmaps.yaml',
-            '/my/tripleo-heat-installer-templates/puppet/foo.yaml',
-            '/my/tripleo-heat-installer-templates//docker/bar.yaml',
-            '/my/tripleo-heat-installer-templates/notouch.yaml',
-            '/my/tripleo-heat-installer-templates/custom.yaml',
-            '/my/tripleo-heat-installer-templates/something.yaml',
-            '/my/tripleo-heat-installer-templates/outside.yaml',
-            'foo.yaml']
+            os.path.join(tht_render, plan_env_env_name),
+            os.path.join(tht_render, 'passwords.yaml'),
+            os.path.join(tht_render,
+                         'environments/config-download-environment.yaml'),
+            os.path.join(tht_render,
+                         'environments/deployed-server-noop-ctlplane.yaml'),
+            os.path.join(tht_render,
+                         'tripleoclient-hosts-portmaps.yaml'),
+            'hiera_or.yaml',
+            os.path.join(tht_render, 'standalone-stack-vstate-dropin.yaml'),
+            os.path.join(tht_render, 'foo.yaml'),
+            os.path.join(tht_render, 'outside.yaml')]
 
-        environment = self.cmd._setup_heat_environments(parsed_args)
+        with mock.patch('os.path.isfile'):
+            environment = self.cmd._setup_heat_environments(parsed_args)
 
-        self.assertEqual(environment, expected_env)
+            self.assertEqual(expected_env, environment)
 
     @mock.patch('tripleoclient.v1.tripleo_deploy.Deploy.'
                 '_create_working_dirs', autospec=True)
@@ -500,6 +604,8 @@ class TestDeployUndercloud(TestPluginV1):
             env
         )
 
+    @mock.patch('os.mkdir')
+    @mock.patch('six.moves.builtins.open')
     @mock.patch('tripleoclient.v1.tripleo_deploy.Deploy.'
                 '_populate_templates_dir')
     @mock.patch('tripleoclient.v1.tripleo_deploy.Deploy.'
@@ -534,7 +640,8 @@ class TestDeployUndercloud(TestPluginV1):
                                     mock_launchheat, mock_download, mock_tht,
                                     mock_wait_for_port, mock_createdirs,
                                     mock_cleanupdirs, mock_launchansible,
-                                    mock_tarball, mock_templates_dir):
+                                    mock_tarball, mock_templates_dir,
+                                    mock_open, mock_os):
 
         parsed_args = self.check_parser(self.cmd,
                                         ['--local-ip', '127.0.0.1',
