@@ -378,6 +378,12 @@ def get_role_net_hostname_map(stack):
             return output['output_value']
 
 
+def get_role_net_ip_map(stack):
+    for output in stack.to_dict().get('outputs', {}):
+        if output['output_key'] == 'RoleNetIpMap':
+            return output['output_value']
+
+
 def get_hosts_entry(stack):
     for output in stack.to_dict().get('outputs', {}):
         if output['output_key'] == 'HostsEntry':
@@ -1133,3 +1139,79 @@ def build_prepare_env(environment_files, environment_directories):
             object_request=get_env_file))
 
     return env
+
+
+def rel_or_abs_path(tht_root, file_path):
+    '''Find a file, either absolute path or relative to the t-h-t dir'''
+    path = os.path.abspath(file_path)
+    if not os.path.exists(path):
+        path = os.path.abspath(os.path.join(tht_root, file_path))
+    if not os.path.exists(path):
+        raise exceptions.DeploymentError(
+            "Can't find path %s %s" % (file_path, path))
+    return path
+
+
+def load_config(osloconf, path):
+    '''Load oslo config from a file path. '''
+    log = logging.getLogger(__name__ + ".load_config")
+    conf_params = []
+    if os.path.isfile(path):
+        conf_params += ['--config-file', path]
+    else:
+        log.warning(_('%s does not exist. Using defaults.') % path)
+    osloconf(conf_params)
+
+
+def configure_logging(log, level, log_file):
+    '''Mimic oslo_log default levels and formatting for the logger. '''
+    fhandler = logging.FileHandler(log_file)
+    formatter = logging.Formatter(
+        '%(asctime)s.%(msecs)03d %(process)d %(levelname)s '
+        '%(name)s [  ] %(message)s',
+        '%Y-%m-%d %H:%M:%S')
+
+    if level > 1:
+        log.setLevel(logging.DEBUG)
+        fhandler.setLevel(logging.DEBUG)
+    else:
+        # NOTE(bogdando): we are making an exception to the oslo_log'ish
+        # default WARN level to have INFO logs as well. Some modules
+        # produce INFO msgs we want to see and keep by default, like
+        # pre-flight valiation notes.
+        log.setLevel(logging.INFO)
+        fhandler.setLevel(logging.INFO)
+
+    fhandler.setFormatter(formatter)
+    log.addHandler(fhandler)
+
+
+def _name_helper(basename, arch=None):
+    if arch:
+        basename = arch + '-' + basename
+    return basename
+
+
+def overcloud_kernel(basename, arch=None):
+    return (_name_helper('%s-vmlinuz' % basename, arch=arch),
+            '.vmlinuz')
+
+
+def overcloud_ramdisk(basename, arch=None):
+    return (_name_helper('%s-initrd' % basename, arch=arch),
+            '.initrd')
+
+
+def overcloud_image(basename, arch=None):
+    return (_name_helper(basename, arch=arch),
+            '.qcow2')
+
+
+def deploy_kernel(arch=None):
+    return (_name_helper('bm-deploy-kernel', arch=arch),
+            '.kernel')
+
+
+def deploy_ramdisk(arch=None):
+    return (_name_helper('bm-deploy-ramdisk', arch=arch),
+            '.initramfs')
