@@ -21,7 +21,10 @@ import subprocess
 
 from openstackclient.i18n import _
 
+from oslo_config import cfg
+
 from tripleoclient import command
+from tripleoclient import constants
 from tripleoclient import utils
 from tripleoclient.v1 import undercloud_config
 
@@ -31,6 +34,7 @@ class InstallUndercloud(command.Command):
 
     auth_required = False
     log = logging.getLogger(__name__ + ".InstallUndercloud")
+    osloconfig = cfg.CONF
 
     def get_parser(self, prog_name):
         parser = argparse.ArgumentParser(
@@ -41,10 +45,22 @@ class InstallUndercloud(command.Command):
         parser.add_argument(
             '--use-heat',
             dest='use_heat',
-            action='store_true',
-            default=False,
-            help=_("Perform undercloud deploy using heat"),
+            nargs='?',
+            default=None,
+            const="true",
+            help=_('This option is deprecated in Rocky. It makes sure that we '
+                   'perform undercloud deploy using ephemeral '
+                   '(one-time create and forget) heat stack and ansible.'),
         )
+        parser.add_argument('--force-stack-update',
+                            dest='force_stack_update',
+                            action='store_true',
+                            default=False,
+                            help=_("Do a virtual update of the ephemeral "
+                                   "heat stack. New or failed deployments "
+                                   "always have the stack_action=CREATE. This "
+                                   "option enforces stack_action=UPDATE."),
+                            )
         parser.add_argument(
             '--no-validations',
             dest='no_validations',
@@ -65,22 +81,35 @@ class InstallUndercloud(command.Command):
         return parser
 
     def take_action(self, parsed_args):
+        # Fetch configuration used to add logging to a file
+        utils.load_config(self.osloconfig, constants.UNDERCLOUD_CONF_PATH)
+        utils.configure_logging(self.log, self.app_args.verbose_level,
+                                self.osloconfig['undercloud_log_file'])
         self.log.debug("take_action(%s)" % parsed_args)
 
         utils.ensure_run_as_normal_user()
-        if parsed_args.use_heat:
+        if parsed_args.use_heat is not None:
+            self.log.warning('--use-heat is deprecated in Rocky')
+        # NOTE(EmilienM): For backwards compatibility until CI has been
+        # switched we need to still run instack-undercloud when --use-heat
+        # is not provided. This will be removed in a follow up patch
+        # once CI has been converted to pass in --use-heat=False
+        if parsed_args.use_heat is None or \
+                parsed_args.use_heat.lower() == "false":
+            self.log.warning(_('Non-containerized undercloud deployment is '
+                             'deprecated in Rocky cycle.'))
+            cmd = ["instack-install-undercloud"]
+        else:
             no_validations = parsed_args.dry_run or parsed_args.no_validations
             cmd = undercloud_config.\
                 prepare_undercloud_deploy(
                     no_validations=no_validations,
-                    verbose_level=self.app_args.verbose_level)
-        else:
-            self.log.warning(_('Non-containerized undercloud deployment is '
-                             'deprecated in Rocky cycle.'))
-            cmd = ["instack-install-undercloud"]
-        if parsed_args.dry_run:
-            print(' '.join(cmd))
-        else:
+                    verbose_level=self.app_args.verbose_level,
+                    force_stack_update=parsed_args.force_stack_update,
+                    dry_run=parsed_args.dry_run)
+
+        self.log.warning("Running: %s" % ' '.join(cmd))
+        if not parsed_args.dry_run:
             subprocess.check_call(cmd)
 
 
@@ -89,22 +118,24 @@ class UpgradeUndercloud(InstallUndercloud):
 
     auth_required = False
     log = logging.getLogger(__name__ + ".UpgradeUndercloud")
+    osloconfig = cfg.CONF
 
     def take_action(self, parsed_args):
+        # Fetch configuration used to add logging to a file
+        utils.load_config(self.osloconfig, constants.UNDERCLOUD_CONF_PATH)
+        utils.configure_logging(self.log, self.app_args.verbose_level,
+                                self.osloconfig['undercloud_log_file'])
         self.log.debug("take action(%s)" % parsed_args)
 
         utils.ensure_run_as_normal_user()
-        if parsed_args.use_heat:
-            cmd = undercloud_config.\
-                prepare_undercloud_deploy(
-                    upgrade=True,
-                    yes=parsed_args.yes,
-                    no_validations=parsed_args.
-                    no_validations,
-                    verbose_level=self.app_args.verbose_level)
-            print("Running: %s" % ' '.join(cmd))
-            subprocess.check_call(cmd)
-        else:
+        if parsed_args.use_heat is not None:
+            self.log.warning('--use-heat is deprecated in Rocky')
+        # NOTE(EmilienM): For backwards compatibility until CI has been
+        # switched we need to still run instack-undercloud when --use-heat
+        # is not provided. This will be removed in a follow up patch
+        # once CI has been converted to pass in --use-heat=False
+        if parsed_args.use_heat is None or \
+                parsed_args.use_heat.lower() == "false":
             self.log.warning(_('Non-containerized undercloud deployment is '
                              'deprecated in Rocky cycle.'))
             subprocess.check_call(['sudo', 'yum', 'update', '-y',
@@ -115,3 +146,14 @@ class UpgradeUndercloud(InstallUndercloud):
             # https://bugzilla.redhat.com/show_bug.cgi?id=1315467
             subprocess.check_call(['sudo', 'systemctl', 'restart',
                                   'openstack-nova-api'])
+        else:
+            cmd = undercloud_config.\
+                prepare_undercloud_deploy(
+                    upgrade=True,
+                    yes=parsed_args.yes,
+                    no_validations=parsed_args.
+                    no_validations,
+                    verbose_level=self.app_args.verbose_level,
+                    force_stack_update=parsed_args.force_stack_update)
+            self.log.warning("Running: %s" % ' '.join(cmd))
+            subprocess.check_call(cmd)
